@@ -16,7 +16,7 @@ use kitsune_p2p_types::agent_info::{AgentInfoInner, AgentInfoSigned};
 use kitsune_p2p_types::bin_types::*;
 use kitsune_p2p_types::config::KitsuneP2pTuningParams;
 use kitsune_p2p_types::dht_arc::loc8::Loc8;
-use kitsune_p2p_types::dht_arc::{ArcInterval, DhtArc, DhtLocation};
+use kitsune_p2p_types::dht_arc::{DhtArc, DhtArcRange, DhtLocation};
 use kitsune_p2p_types::metrics::metric_task;
 use kitsune_p2p_types::tx2::tx2_api::*;
 use kitsune_p2p_types::tx2::tx2_pool_promote::*;
@@ -225,7 +225,7 @@ impl SwitchboardState {
             self.space.clone(),
             node_ep,
             agent.clone(),
-            initial_arc.canonical(),
+            DhtArc::from_parts(initial_arc.canonical(), loc8.into()),
         );
         if let Some(existing) = self.local_agent_by_loc8(loc8) {
             panic!(
@@ -276,9 +276,9 @@ impl SwitchboardState {
             let node_id = ep.uniq();
             let ascii = if with_ops {
                 let ops = node.ops.keys().copied();
-                ArcInterval::Empty.to_ascii_with_ops(width, ops)
+                DhtArcRange::Empty.to_ascii_with_ops(width, ops)
             } else {
-                ArcInterval::Empty.to_ascii(width)
+                DhtArcRange::Empty.to_ascii(width)
             };
             println!(
                 "{:>4} {:>+5} ({:^width$})",
@@ -288,14 +288,14 @@ impl SwitchboardState {
                 width = width
             );
             for (agent_loc8, agent) in node.local_agents.iter() {
-                let interval = agent.info.storage_arc.interval();
+                let interval = &agent.info.storage_arc;
                 let ascii = interval.to_ascii(width);
                 println!(
                     "{:>4} {:>+5} |{:^width$}| {:>+4} {:?}",
                     "",
                     agent_loc8,
                     ascii,
-                    interval.center_loc().as_loc8(),
+                    interval.start_loc().as_loc8(),
                     interval.map(|b| DhtLocation::as_loc8(&b)),
                     width = width
                 );
@@ -522,21 +522,21 @@ impl SwitchboardState {
 /// The reason for this type is to reduce the redundancy
 /// of needing to specify both the agent's location and the
 /// storage arc. It is most convenient to specify arcs in terms
-/// of ArcInterval, but we use the agent's Loc8 location as their
+/// of DhtArcRange, but we use the agent's Loc8 location as their
 /// unique identifier. This allows specification of agents in
 /// terms of arcs.
 #[derive(Clone, derive_more::AsRef)]
 pub struct SwitchboardAgent {
     pub(super) loc: Loc8,
-    initial_arc: ArcInterval<Loc8>,
+    initial_arc: DhtArcRange<Loc8>,
 }
 
 impl SwitchboardAgent {
     /// Construct an agent with a full arc at the specified location.
-    pub fn full<L: Into<Loc8>>(loc: L) -> Self {
+    pub fn full<L: Copy + Into<Loc8>>(loc: L) -> Self {
         Self {
             loc: loc.into(),
-            initial_arc: ArcInterval::Full,
+            initial_arc: DhtArcRange::Full,
         }
     }
 
@@ -545,8 +545,8 @@ impl SwitchboardAgent {
     pub fn from_bounds<L: Into<Loc8>>(lo: L, hi: L) -> Self {
         let lo: Loc8 = lo.into();
         let hi: Loc8 = hi.into();
-        let initial_arc = ArcInterval::Bounded(lo, hi);
-        let loc8 = initial_arc.clone().canonical().center_loc().as_loc8();
+        let initial_arc = DhtArcRange::Bounded(lo, hi);
+        let loc8 = lo;
 
         Self {
             loc: loc8,
@@ -556,13 +556,18 @@ impl SwitchboardAgent {
 
     /// Construct an agent from arc bounds.
     /// The agent's location is taken as the midpoint of the arc.
-    pub fn from_center_and_half_len<L: Into<Loc8>>(center: L, half_len: u8) -> Self {
-        let center: Loc8 = center.into();
-        let half_len = Loc8::upscale(half_len as i32);
-        let initial_arc = DhtArc::new(center, half_len).interval().as_loc8();
+    pub fn from_start_and_half_len<L: Into<Loc8>>(start: L, half_len: u8) -> Self {
+        let start: Loc8 = start.into();
+        let initial_arc = if half_len == 0 {
+            DhtArcRange::Empty
+        } else {
+            let len = half_len * 2 - 1;
+            let end = Loc8::from((start.as_u8().wrapping_add(len).wrapping_sub(1)) as i8);
+            DhtArcRange::Bounded(start, end)
+        };
 
         Self {
-            loc: center,
+            loc: start,
             initial_arc,
         }
     }
@@ -679,7 +684,7 @@ fn fake_agent_info(
     space: KSpace,
     node: &NodeEp,
     agent: KAgent,
-    interval: ArcInterval,
+    interval: DhtArc,
 ) -> AgentInfoSigned {
     use crate::fixt::*;
     let url_list = vec![node.local_addr().unwrap()];
@@ -704,7 +709,7 @@ fn fake_agent_info(
     let state = AgentInfoInner {
         space,
         agent,
-        storage_arc: DhtArc::from_interval(interval),
+        storage_arc: interval,
         url_list,
         signed_at_ms: 0,
         expires_at_ms: u64::MAX,
